#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

#include "light_ws2812.h"
#include "pov.h"


extern void ws2812_setleds(struct cRGB*, unsigned int);


#define OCR1A_MAX 5000
#define MAX_STEPS 128
#define BREAK_OFFSET 20
#define ACCELERATION 5
#define OCR1A_MIN 1900


unsigned char (*image[10])[65][NUM_LEDS];

volatile int  n_images = 2;


struct cRGB leds_array[8];



void set_color(cRGB *led, unsigned char color_code)
{
	switch (color_code)
	{
		case COLOR_NONE:
			led->r = 0;
			led->g = 0;
			led->b = 0;
			break;
			
		case COLOR_RED:
			led->r = 255;
			led->g = 0;
			led->b = 0;
			break;
			
		case COLOR_GREEN:
			led->r = 0;
			led->g = 255;
			led->b = 0;
			break;
			
		case COLOR_BLUE:
			led->r = 0;
			led->g = 0;
			led->b = 255;
			break;
			
		case COLOR_YELLOW:
			led->r = 255;
			led->g = 255;
			led->b = 0;
			break;
			
		case COLOR_MAUVE:
			led->r = 255;
			led->g = 0;
			led->b = 255;
			break;
			
		case COLOR_CYAN:
			led->r = 0;
			led->g = 255;
			led->b = 255;
			break;
			
		case COLOR_WHITE:
			led->r = 255;
			led->g = 255;
			led->b = 255;
			break;
	}
	
}

void show_line(unsigned char line[])
{
	for (unsigned char i = 0; i < NUM_LEDS; i++)
		set_color(leds_array + i, line[i]);

	ws2812_setleds(leds_array, NUM_LEDS);
}

volatile int steps = 0;
volatile char direction = 0;

volatile char line_index = 7;
volatile unsigned char asc = 0;

volatile unsigned char image_index = 0;

volatile char pressed = 0;

// intrerupere buton
ISR(PCINT1_vect)
{
		
	if ((PIND & (1 << PD6)) == 0)
	{
        _delay_ms(40);
        if ((PIND & (1 << PD6)) == 0) {
            image_index = (image_index + 1) % n_images;
        }
    }
		
	
}


// intrerupere timer1
ISR(TIMER1_COMPA_vect)
{	
	PORTA ^= (1 << PA0);

	if (direction == 0)
	{

		if (steps < MAX_STEPS-BREAK_OFFSET)
			if (OCR1A - (steps*steps/40 + 40) > OCR1A_MIN)
				OCR1A -= (steps*steps/40 + 40);
			else
				OCR1A = OCR1A_MIN;
		else
		{
			OCR1A += ACCELERATION * (BREAK_OFFSET - (MAX_STEPS - steps)) * (BREAK_OFFSET - (MAX_STEPS - steps));
		}
		
		if (steps + 1 <= MAX_STEPS)
			steps++;

		if (steps == MAX_STEPS)
		{
			direction = 1;
			PORTA |= (1 << PA1);
			
			OCR1A = OCR1A_MAX;
		}
		
	}
	else
	{
		if (steps - BREAK_OFFSET > 0)
			if (OCR1A - ((MAX_STEPS - steps)*(MAX_STEPS - steps)/40 + 40) > OCR1A_MIN)
				OCR1A -= ((MAX_STEPS - steps)*(MAX_STEPS - steps)/40 + 40);
			else
				OCR1A = OCR1A_MIN;
		else
		{
			OCR1A += ACCELERATION * (BREAK_OFFSET - steps) * (BREAK_OFFSET - steps);
		}
		
		if (steps - 1 >= 0)
			steps--;
		
		if (steps == 0)
		{
			direction = 0;
			PORTA &= ~(1 << PA1);
			
			OCR1A = OCR1A_MAX;
		}
	}
	
	show_line((*image[image_index])[steps / 2]);
	
}



int main(void)
{

	// setare imagini
	image[0] = &image1;
	image[1] = &image2;
	n_images = 2;


	//led
	DDRD |= (1 << PD7);
	
	// buton pt schimbare text
	DDRD &= ~(1 << PD6);
	PCICR  = (1 << PCIE1);
    PCMSK1 = (1 << PCINT30);


	// step
	DDRA |= (1 << PA0);
	
	// directie
	DDRA |= (1 << PA1);

	// dout neopixel array
	DDRC |= (1 << PC0);
	
	// timer 1 (stepper)
	OCR1A = OCR1A_MAX;
	TIMSK1 = (1 << OCIE1A);
	TCCR1B = (2 << CS10) | (1 << WGM12);


	

	sei();

	
	
	while(1)
	{
		
	}
	
	return 0;
}

